@@ -26,16 +26,8 @@ from StringIO import StringIO
 from httpexceptor import HTTP400
 
 
-def get_form(environ):
-    try:
-        form = {
-            'application/x-www-form-urlencoded': environ['tiddlyweb.query'],
-            'multipart/form-data': FieldStorage(fp=environ['wsgi.input'], environ=environ)
-        }
-    except (ValueError, IOError), exc:
-        raise HTTP400('The was a problem with your syntax. Please check and try again')
+LOGGER = logging.getLogger(__name__)
 
-    return form.get(environ['tiddlyweb.type'])
 
 def retrieve_item(obj, key):
     if getattr(obj, 'getfirst', None):
@@ -43,6 +35,7 @@ def retrieve_item(obj, key):
     else:
         return obj[key][0]
 
+
 def post_tiddler_to_container(environ, start_response):
     """
     entry point for recipes/foo/tiddlers or bags/foo/tiddlers
@@ -50,20 +43,18 @@ def post_tiddler_to_container(environ, start_response):
     we have included the tiddler name in the form,
     so get that and carry on as normal
     """
-    try:
-        form = get_form(environ)
-    except timeout:
-        return []
+    form = environ['tiddlyweb.query']
 
     def get_name():
         if 'title' in form:
             return retrieve_item(form, 'title')
         else:
-            return form['file'].filename
+            files = environ['tiddlyweb.input_files']
+            return files[0].filename
 
     try:
         tiddler_name = urllib.quote(get_name())
-    except KeyError:
+    except (KeyError, IndexError):
         tiddler_name = str(uuid4())
 
     Serialization.form = form
@@ -103,10 +94,11 @@ class Serialization(SerializationInterface):
         nb: input_string is ignored. You need to set Serialization.form
         to the form object prior to calling.
         """
-        if not getattr(self, 'form', None):
+
+        if not hasattr(self, 'form'):
             raise NoSerializationError('Form expected, but none found')
-        if 'file' in self.form and getattr(self.form['file'], 'file', None):
-            my_file = self.form['file']
+        if self.environ.get('tiddlyweb.input_files'):
+            my_file = self.environ['tiddlyweb.input_files'][0]
             if not my_file.file: raise TiddlerFormatError
             tiddler.type = my_file.type
             tiddler.text = my_file.file.read()
@@ -121,6 +113,8 @@ class Serialization(SerializationInterface):
         else:
             keys = ['created', 'modified', 'modifier', 'text']
             for key in self.form:
+                if key == 'title':
+                    continue
                 if key in keys:
                     setattr(tiddler, key, retrieve_item(self.form, key))
                 elif key == 'tags':
@@ -161,7 +155,7 @@ def update_handler(selector, path, new_handler, server_prefix):
             selector.mappings[index] = (regex, handler)
             return
 
-    logging.debug('%s not found in URL mapping. Not replaced' % path)
+    LOGGER.debug('%s not found in URL mapping. Not replaced' % path)
 
 def init(config):
     """
@@ -180,7 +174,6 @@ def init(config):
     update_handler(selector, '/bags/{bag_name:segment}/tiddlers[.{format}]',
         dict(POST=post_tiddler_to_container), config.get('server_prefix', ''))
 
-    config['extension_types']['form'] = 'application/x-www-form-urlencoded'
     config['serializers']['application/x-www-form-urlencoded'] = \
         ['tiddlywebplugins.form', 'application/x-www-form-urlencoded; charset=UTF-8']
     config['serializers']['multipart/form-data'] = \
